(*************************************************************************)
(*                                                                       *)
(*                                OCaPIC                                 *)
(*                                                                       *)
(*            Benoit Vaugon, Université Pierre et Marie Curie            *)
(*                                                                       *)
(*    Ce fichier est distribué sous les termes de la licence CeCILL-B    *)
(*    décrite dans le fichier ../LICENCE.                                *)
(*                                                                       *)
(*************************************************************************)

type bus_size = Four | Eight

module type P =
sig
  type reg
  type bit
  val read_reg : reg -> int
  val write_reg : reg -> int -> unit
  val set_bit : bit -> unit
  val clear_bit : bit -> unit
  val test_bit : bit -> bool
  val tris_of_port : reg -> reg
  val tris_of_pin : bit -> bit
end

module type S =
sig
  module Pic : P
  val bus_size : bus_size
  val e : Pic.bit
  val rs : Pic.bit
  val rw : Pic.bit
  val port : Pic.reg
end

module Connect (C : S) =
struct
  open C
  open Pic

  let tris = tris_of_port port;;

  (***)

  let send =
    if bus_size <> Four then
      fun c ->
	clear_bit rw;
	write_reg tris 0x00;
	write_reg port c;
	set_bit e;
	clear_bit e;
    else
      fun c ->
	clear_bit rw;
	write_reg tris ((read_reg tris) land 0x0F);
	let oldp = read_reg port land 0x0F in
	write_reg port ((c land 0xF0) lor oldp);
	set_bit e;
	clear_bit e;
	write_reg port (((c lsl 4) land 0xF0) lor oldp);
	set_bit e;
	clear_bit e;;

  let receive =
    if bus_size <> Four then
      fun () ->
	write_reg tris 0xFF;
	set_bit rw;
	set_bit e;
	let res = read_reg port in
	clear_bit e;
	res
    else
      fun () ->
	write_reg tris ((read_reg tris) lor 0xF0);
	set_bit rw;
	set_bit e;
	let tmp = read_reg port land 0xF0 in
	clear_bit e;
	set_bit e;
	let res = (read_reg port lsr 4) lor tmp in
	clear_bit e;
	res;;

  let send_data d = set_bit rs ; send d;;

  let send_instr i = clear_bit rs ; send i;;

  let wait_ready () =
    if bus_size <> Four then write_reg tris 0xFF
    else write_reg tris ((read_reg tris) lor 0xF0);
    clear_bit rs;
    set_bit rw;
    set_bit e;
    while read_reg port land 0b10000000 <> 0 do () done;
    clear_bit e;
  ;;

  (***)

  type direction = Left | Right
  type mode = Cursor_left | Cursor_right | Display_left | Display_right
  type state = On | Off
  type cursor = Invisible | Underscore | Block
  type font = F5x8 | F5x10
  type lmode = One | Two
  
  (***)

  let clear () = wait_ready () ; send_instr 0b1 ;;
  
  let home () = wait_ready () ; send_instr 0b10 ;;

  (***)

  let init =
    if bus_size <> Four then
      fun () ->
	clear_bit (tris_of_pin e);
	clear_bit (tris_of_pin rs);
	clear_bit (tris_of_pin rw);
	wait_ready ();
	clear ();
	home ();
    else
      fun () ->
	clear_bit (tris_of_pin e);
	clear_bit (tris_of_pin rs);
	clear_bit (tris_of_pin rw);
	wait_ready ();
	clear_bit rs;
	clear_bit rw;
	write_reg tris ((read_reg tris) land 0x0F);
	write_reg port (0x20 lor ((read_reg port) land 0x0F));
	set_bit e;
	clear_bit e;
	clear ();
	home ();;

  let config ?(mode=Cursor_right) ?(disp=On) ?(cursor=Invisible)
      ?(lmode=Two) ?(font=F5x8) () =
    wait_ready ();
    send_instr (
      match mode with
	| Cursor_left -> 0b100
	| Cursor_right -> 0b110
	| Display_left -> 0b111
	| Display_right -> 0b101
    );
    wait_ready ();
    send_instr (
	(if disp = On then 0b1100 else 0b1000) lor
	(match cursor with
	   | Invisible -> 0b1000
	   | Underscore -> 0b1010
	   | Block -> 0b1011)
    );
    wait_ready ();
    send_instr (
      (if bus_size <> Four then 0b110000 else 0b100000) lor
      (if lmode = Two then 0b101000 else 0b100000) lor
      (if font = F5x10 then 0b100100 else 0b100000)
    );
  ;;
  
  (***)

  let current_position () =
    wait_ready ();
    clear_bit rs;
    let p = receive () land 0b01111111 in
    if p >= 0x54 then (4, p - 0x54) else if p >= 0x40 then (2, p - 0x40)
    else if p >= 0x14 then (3, p - 0x14) else (1, p);;

  let moveto line column =
    let ofs = match line with 2 -> 0x40 | 3 -> 0x14 | 4 -> 0x54 | _ -> 0x00 in
    wait_ready ();
    send_instr (0b10000000 lor (ofs + column));;

  let shift_cursor d =
    wait_ready ();
    send_instr (if d = Left then 0b10000 else 0b10100);;
  
  let shift_display d =
    wait_ready ();
    send_instr (if d = Left then 0b11000 else 0b11100);;

  (***)

  let print_char c =
    if c = '\n' then begin
      let (li, _) = current_position () in
	moveto (succ li) 0;
    end else begin
      wait_ready ();
      send_data (int_of_char c)
    end;;

  let print_string str =
    for i = 0 to String.length str - 1 do
      print_char str.[i]
    done;;

  let print_int i = print_string (string_of_int i);;

  let output : out_channel = Obj.magic print_char;;

end
